package Prism.benchmark;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;

import Prism.core.*;
import Prism.extensions.port.ExtensiblePort;
import Prism.extensions.port.distribution.SocketDistribution;

public class DeepDistBench {

	public static void main(String[] args) 
	{
		Runtime rt = Runtime.getRuntime();
		System.gc();
		long initMemory1 = rt.freeMemory();
		System.out.println("Initial memory " + initMemory1);
		long totalMemory1 = rt.totalMemory();
		System.out.println("Initial total memory " + totalMemory1);
		
		String ip = args[1];
		int numOfComps = Integer.valueOf(args[2]).intValue();
		int numOfEvents = Integer.valueOf(args[3]).intValue();
		int sizeOfSched = Integer.valueOf(args[4]).intValue();
		int numOfThreads = Integer.valueOf(args[5]).intValue();
		boolean last = false;
		int destPort = 0;
		if (Integer.valueOf(args[6]).intValue() == 1)
		{
			last = true;
		}
		else
			destPort = Integer.valueOf(args[8]).intValue();
		int myPort = Integer.valueOf(args[7]).intValue();
		int numOfArchs = Integer.valueOf(args[9]).intValue();

		
        FIFOScheduler sched = new FIFOScheduler(sizeOfSched);
        Scaffold scaff = new Scaffold();
        RRobinDispatcher disp = new RRobinDispatcher(sched, numOfThreads);
        scaff.dispatcher=disp;
        scaff.scheduler=sched;
        Architecture arch = new Architecture("Bottom_Arch");                
        arch.scaffold=scaff;
		SenderImpl senderImpl = null;
		
		Connector topConn = new Connector("conn");
		ExtensiblePort connTopExtPort = new ExtensiblePort("Ext_Port", PrismConstants.REQUEST);
		SocketDistribution sd = new SocketDistribution(connTopExtPort);	
    	connTopExtPort.addDistributionModule(sd);
    	connTopExtPort.scaffold = scaff;
		topConn.addConnPort(connTopExtPort);					
        arch.add(connTopExtPort);	
        arch.add(topConn);
        
		if (args[0].equalsIgnoreCase("sender"))
		{			
			senderImpl = new SenderImpl(numOfEvents, numOfComps, numOfArchs);
			Component sender = new Component("sender", senderImpl);	
			Port sendPort = new Port("sendPort", PrismConstants.REQUEST);
			sender.scaffold = scaff;
			sender.addCompPort(sendPort);
			arch.add(sender);
			arch.add(sendPort);		
			Port topConnPort = new Port ("Port", PrismConstants.REPLY);
			topConn.addConnPort(topConnPort);
			arch.add(topConnPort);
			arch.weld(topConnPort,sendPort);
		}
		else
		{
			Connector bottomConn = new Connector("conn");			
			ExtensiblePort connBottomExtPort = new ExtensiblePort("Ext_Port", PrismConstants.REPLY);
			sd = new SocketDistribution(connBottomExtPort, myPort);	
	    	connBottomExtPort.addDistributionModule(sd);
	    	connBottomExtPort.scaffold = scaff;
			bottomConn.addConnPort(connBottomExtPort);					
	        arch.add(connBottomExtPort);																			        
	        Component[] comps = new Component[numOfComps];
	        AbstractImplementation[] compsImpls = new AbstractImplementation[numOfComps];      
	        Port [] compsTopPorts = new Port[numOfComps];
	        Port [] compsBottomPorts = new Port[numOfComps];
	        Port [] topConnPorts = new Port[numOfComps];
	        Port [] bottomConnPorts = new Port[numOfComps];
	        for (int i=0; i< comps.length; i++)
	        {                 
                compsImpls[i] = new PropagatorImpl(last);
                comps[i] = new Component("Bottom_Comp", compsImpls[i]);
                comps[i].scaffold=scaff;                
				arch.add(comps[i]);
				
		        compsTopPorts[i] = new Port ("Port", PrismConstants.REQUEST);		    	        	
				comps[i].addCompPort(compsTopPorts[i]);
				arch.add(compsTopPorts[i]);
				
		        compsBottomPorts[i] = new Port ("Port", PrismConstants.REPLY);		    	        	
				comps[i].addCompPort(compsBottomPorts[i]);
				arch.add(compsBottomPorts[i]);		
				
				topConnPorts[i] = new Port ("Port", PrismConstants.REPLY);
				topConn.addConnPort(topConnPorts[i]);
				arch.add(topConnPorts[i]);
				
				bottomConnPorts[i] = new Port ("Port", PrismConstants.REQUEST);
				bottomConn.addConnPort(bottomConnPorts[i]);
				arch.add(bottomConnPorts[i]);
				
				arch.weld(compsTopPorts[i], topConnPorts[i]);
				arch.weld(compsBottomPorts[i], bottomConnPorts[i]);
	        }
		}
		disp.start();
		arch.start();
		if (!last)
			connTopExtPort.connect(ip, destPort);
				
		if (args[0].equalsIgnoreCase("sender"))
		{
		      System.out.println("Tell me when to start by pressing enter "); 
		      //  open up standard input 
		      
		      BufferedReader br = new BufferedReader(new InputStreamReader(System.in)); 
		      //  read the username from the command-line; need to use try/catch with the 
		      //  readLine() method 
		      try { 
		      		br.readLine(); 
		      		
					//-----------------------------------
					// Architecture initialization ends
					//-----------------------------------
					System.gc();
					long remainingMemory = rt.freeMemory();
					System.out.println("Final memory " + remainingMemory);
					long totalMemory2 = rt.totalMemory();
					System.out.println("Final total memory " + totalMemory2);
					System.out.println("Memory used\t" + (initMemory1 - remainingMemory) +" bytes");
					System.out.println("Memory used 2\t" + ((totalMemory2-remainingMemory)-(totalMemory1-initMemory1)) +" bytes");							
					
		      		System.out.println("Starting");
		      		senderImpl.start2();
		      		
		      } catch (IOException ioe) { 
		         System.out.println("IO error trying to read your name!"); 
		         System.exit(1); 
		      } 		
		}
	}
}
