package Prism.benchmark;

import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.net.ServerSocket;
import java.net.Socket;
import Prism.core.*;

class TimeVariables
{
	public long startTime;
	public long endTime;
}

public class JavaTCPIPBenchmark 
{	

	public static void main(String[] args) 
	{	
		Runtime rt = Runtime.getRuntime();
		System.gc();
		long initMemory = rt.freeMemory();
		System.out.println("Initial memory " + initMemory);
		long totalMemory = rt.totalMemory();
		System.out.println("Initial total memory " + totalMemory);

		if (args[0].equals("sender"))
		{
			WritingThread writingThread = new WritingThread (args[2], Integer.valueOf(args[3]).intValue(), Integer.valueOf(args[1]).intValue());
		}
		else
		{
			ListenForConnectionsThread listenThread = new ListenForConnectionsThread (Integer.valueOf(args[3]).intValue(), Integer.valueOf(args[1]).intValue());
			listenThread.start();			
		}
		
		System.gc();
		long remainingMemory = rt.freeMemory();
		System.out.println("Final memory " + remainingMemory);
		totalMemory = rt.totalMemory();
		System.out.println("Final total memory " + totalMemory);
		System.out.println("Memory used\t" + (initMemory - remainingMemory) +" bytes");
	}
			
}

class WritingThread
{
	public WritingThread (String ip, int port, int numEvts)
	{
		try
		{
		    Socket socket = new Socket(ip, port);	
		    ObjectOutputStream writer = new ObjectOutputStream(socket.getOutputStream());
		    Thread.sleep(2000);
            long startTime = System.currentTimeMillis();
            System.out.println("Start time: "+ startTime + " ms");	
		    for (int i=0; i< numEvts; i++)
		    {
		    	Event event = new Event("do");		   
		    	event.eventType = PrismConstants.REQUEST;
		    	//event.addParameter("start_time", new Long(startTime));
		    	writer.writeObject(event);
		    	writer.flush();
		    	writer.reset();
		    }
		}
		catch (Exception e)
		{
			e.getMessage();
			e.printStackTrace();
		}
	}
}

class ReadingThread extends Thread
{
    public boolean keepWorking;
    public Socket socket;
    public ObjectInputStream ir;
    int counter = 0;
    int numEvts;

    
    public ReadingThread(Socket sc, int pNumEvts)
    {
        super();
        keepWorking = true;
        numEvts = pNumEvts;
        try
		{        	
            socket = sc;
            ir = new ObjectInputStream(socket.getInputStream());
		}
        catch (IOException e)
		{
            e.printStackTrace ();
		}
    }
    
    public void run()
    {
        Object eventObject = null;
    	try
    	{
            while (keepWorking)
            {
                eventObject = (Object) ir.readObject();
                Event event = (Event) eventObject;
                counter++;
                if (counter == numEvts)
                {
                	//long startTime = ((Long)event.getParameter("start_time")).longValue();                   
                	long endTime = System.currentTimeMillis();  ;
        		    System.out.println("end time: " + endTime);
        		    //System.out.println("Time taken: "+ (tv.endTime - tv.startTime) + " ms");
                }
            }
        }
	  	catch (Exception e1)
	  	{
	            System.out.println("Caught Exception in ReadingThread " + e1.toString());	            
	  	}	  	
    }        
}

class ListenForConnectionsThread extends Thread
{
    boolean keepWorking;
    ServerSocket ss;
    int port;
    int numEvts;
    
    /** 
     * Simple constructor.
     * @param parent Parent distribution object
     */
    public ListenForConnectionsThread(int pPort, int pNumEvts)
    {
		super();		
		keepWorking = true;	
		port = pPort;
		numEvts = pNumEvts;	
    }
    
    /**
     * Starts the thread that listens for incoming connections.
     */
    public void run() 
    {
    	try 
		{
			ss = new ServerSocket(port);
		}
		catch (Exception e) 
		{
			System.out.println("\n Exception in ServerSocket(): " + e.toString());
		}


		System.out.println("Ready to accept connections");

		Socket sock;
		
		while (keepWorking) 
		{
			try
			{
				sock = ss.accept();
				System.out.println("\n** Connect from: " + sock.getInetAddress() + ":" + sock.getPort());
				ReadingThread reader = new ReadingThread (sock, numEvts);	
				reader.start();
			}
			catch (Exception e) 
			{
				System.out.println("Exception while listening for connections:  " + e.toString());
			}
		 }    
	}
	
}
