package Prism.core;

public abstract class AbstractImplementation 
{
	protected IComponent associatedComp;

	public void send(Event e)
	{
		if (associatedComp != null)
		{
			associatedComp.send(e);
		}
	}

	public void setAssociatedComponent (IComponent comp)
	{
		associatedComp = comp;
	}
	
	public IComponent getAssociatedComponent ()
	{
		return associatedComp;
	}
	
	public void start()
	{		
	}
	
	/**
	 * This method MUST be called when a component initiates a transaction
	 * They are two wait loops:
	 * 1. If the passivating process has been started the component should not start new transaction to avoid starvation.
	 * 2. If the component cannot be activated (i.e. component is passive) should not start new transactions to preserve passive property.
	 */
	public void startTransaction() {
		synchronized(getAssociatedComponent()) {
			getAssociatedComponent().log("AT: " + getAssociatedComponent().getName() + " starting transaction.");
			
	    	while(getAssociatedComponent().isPassivating()) {
	    		try {
	    			getAssociatedComponent().wait();
	    		} catch (InterruptedException e1) {
	    			e1.printStackTrace();
	    		}
	    	}
	    	
	    	while(!getAssociatedComponent().activate()) {
	    		try {
	    			getAssociatedComponent().wait();
	    		} catch (InterruptedException e1) {
	    			e1.printStackTrace();
	    		}
	    	}
		}
	}

	/**
	 * This method MUST be called when the transaction finished
	 */
	public void finishTransaction() {
		getAssociatedComponent().log("AT: " + getAssociatedComponent().getName() + " finishing transaction.");
		
		synchronized(getAssociatedComponent()) {
			getAssociatedComponent().inactivate();
		}
	}
	
	public abstract void handle(Event e);
	
}
