package Prism.core;

import java.util.Vector;

/**
 *  This interface needs to be implemented by any Prism component.
 * 
 *@version 2.0
 *@author USC Soft. Arch. Group. Contact: Sam Malek <A HREF="mailto:malek@usc.edu"> malek@usc.edu </A>
 */
public interface IComponent
{              
  /**
    * This method sends the event up/down the Prism architecture that this component is a
    * part of.
    *
    * @param e		a Event to be sent to the Brick above/below.
    */ 
    public void send(Event e);
	
  /**
    * Handling of Event. This is application specific code and should be implemented by subclasses of this class.
    *
    * @param e	Event to be handled
    */ 
    public void handle(Event e);	
    
   /**
    * Adds a port. 
    * @param port    Port to be added.
    */   
   public abstract void addCompPort(IPort b);
   
   /**
    * Removes a port.
    *@param port    Port to be removed.
    */   
   public abstract void removeCompPort(IPort b);

   /**
    * This method returns a list of all available ports.
    *@return DynamicArray   List of available ports  
    */   
   public abstract Vector getCompPorts();

   /**
    * Actiavtes the compontes
    * @return if it could get activated
    */
   public abstract boolean activate();

   /**
    * Inactivates the component
    */
   public abstract void inactivate();

   /**
    * Puts the componet in passive state
    * @return if it could get passivated
    */
   public abstract boolean passivate();

   /**
    * Reactivates the component after being passive
    */
   public abstract void reactivate();
   
   /**
    * Flags the start of passivating process
    */
   public abstract void setPassivating();
   
   /**
    * Returns the status of the component
    * @return stutus of the component
    */
   public abstract int getStatus();
   
   /**
    * Uses architecture logging facility to log things.
    * @param text the log
    */
   public abstract void log(String text);
   
   /**
    * Uses architecture logging facility to log info. Two level higher than log.
    * @param text the info
    */
   public abstract void info(String text);
   
   /**
    * If the passivating process has been satrted
    * @return start of passivating process
    */
   public abstract boolean isPassivating();
   
   public abstract String getName();
   
   public abstract float getUsageRatio();
    
   public static final int CONNS_PER_COMPONENT=10;
}
