package Prism.extensions.architecture;

import java.util.Vector;

import Prism.core.*;
import Prism.extensions.architecture.adaptation.AbstractAdaptationModule;
import Prism.extensions.architecture.topology.*;


public class ExtensibleArchitecture extends Architecture 
{
	private AbstractTopology topology;
	private AbstractAdaptationModule adaptationModule;

	public ExtensibleArchitecture(String str) 
	{
	  super(str); 
	}	
	
	public ExtensibleArchitecture(String str, AbstractTopology pTopology, AbstractAdaptationModule pAdaptationModule)
	{
		super(str);
		
		topology = pTopology;
		adaptationModule = pAdaptationModule;
	}
	
	public ExtensibleArchitecture (String str, int pStyle)
	{
		super(str);
		style = pStyle;
	}
	
	public ExtensibleArchitecture(String str, AbstractTopology pTopology, AbstractAdaptationModule pAdaptationModule, int pStyle)
	{
		this(str, pStyle);
		
		topology = pTopology;		
		adaptationModule = pAdaptationModule;
	}	
	
	public void topologyWeld(Brick b1, Brick b2) 
	{
		try
		{
			if (topology != null)
			{
				topology.weld(b1, b2);
			}
		}
		catch (PrismException e)
		{
				System.err.println(e.getMessage());

		}
	}

    // for now this method only works for component replacement and assumes only local architecture
    synchronized public void replace (Brick b, Brick c)
    {    	
    	logger.debug("passivating everything below the component");
    	long replaceStartTime = System.currentTimeMillis();

    	if(c instanceof IComponent) {
			IComponent ic = (IComponent) c;
			
			if(!ic.passivate()) {
	    		logger.warn("The component being replaced should not be ACTIVE");
	    		return;
			}
		}		
    	
    	if (passivateThePassivateSet(b))
    	{    	    		
        	logger.info("Finished Passivating " + b.name + " in: " + (System.currentTimeMillis() - replaceStartTime));
    		logger.debug("quiescing and removing the component that is being replaced");

    		//implies that b is QUIESCENT;    		    		
    		this.remove(b);

    		logger.debug("adding the ports of the old component to the new one");
    		
    		Vector bCompPorts = ((Component)b).getCompPorts();
    		
    		while (bCompPorts.size() != 0)
    		{
    			Port thisPort = (Port)bCompPorts.elementAt(0);
    			((Component)b).removeCompPort(thisPort);
    			((Component)c).addCompPort(thisPort);
    		}
    		
    		logger.debug("adding the new component to the architecture");
    		
    		this.add(c);
    		
    		if (revive(c))
    			logger.debug("the system restarted");
    		else
    			logger.error("there was a problem restarting the system");	
    	}
    	else 
    		logger.error("there was a problem passivating the components");
    	
    }
    
    synchronized public boolean passivateThePassivateSet (Brick b)
    {
		if (adaptationModule != null)
			return adaptationModule.establishPassiveSet(b);
		else
			return false;
    }
    
    synchronized public boolean revive (Brick b)
    {
		if (adaptationModule != null)
	    	return adaptationModule.revive(b);
		else
			return false;
    }	
}
