package Prism.extensions.architecture.adaptation;

import java.util.Vector;

import org.apache.log4j.Logger;

import Prism.core.Brick;
import Prism.core.Component;
import Prism.core.Connector;
import Prism.core.IComponent;
import Prism.core.Port;
import Prism.core.PrismConstants;

public class C2AdaptationModule extends AbstractAdaptationModule {

	public C2AdaptationModule(Logger logger) {
		super(logger);
	}

	@Override
	public boolean establishPassiveSet(Brick b) {
    	Vector bPorts = null;
    	
    	if (b instanceof Component)
    		 bPorts = ((Component)b).getCompPorts();
    	else if (b instanceof Connector)
    		bPorts = ((Connector)b).getConnPorts();
    	
    	logger.debug("passivating component/connector: " + b.name);

    	for (int i=0; i<bPorts.size(); i++)
    	{
    		Port p = (Port) bPorts.elementAt(i);
    		
    		if (p.getPortType() == PrismConstants.REPLY)
    		{
    			Port mutualPort = (Port) p.getMutualPort();
    			Brick belowBrick = mutualPort.getParentBrick();
    			
    			if (!establishPassiveSet (belowBrick)) {
    	    		logger.error("there was a problem passivating below the component: " + b.name);
    	    		return false;
    			}
    		}
    	}
		
    	if(b instanceof IComponent) {
    		synchronized(b) {
	    		IComponent ic = (IComponent) b;
	    		ic.setPassivating();
	    		
		   		while (!ic.passivate()) {
	    			logger.debug("Component/Connector: " + b.name + " is currently active");
	    			
	   				try {
						ic.wait();
					} catch (InterruptedException e) {
						e.printStackTrace();
					}
		   		}
		   		
				logger.debug("Component/Connector: " + b.name + " is now passive");
    		}
    	}
    	
    	return true;
	}

	@Override
	public boolean revive(Brick b) {
    	Vector bPorts = null;
    	if (b instanceof Component)
    		 bPorts = ((Component)b).getCompPorts();
    	else if (b instanceof Connector)
    		bPorts = ((Connector)b).getConnPorts();
    	
    	logger.debug("reviving component/connector: " + b.name);

    	for (int i=0; i<bPorts.size(); i++)
    	{
    		Port p = (Port) bPorts.elementAt(i);
    		if (p.getPortType() == PrismConstants.REPLY)
    		{
    			Port mutualPort = (Port) p.getMutualPort();
    			Brick belowBrick = mutualPort.getParentBrick();
    			
    			if(!revive (belowBrick)) {
        			logger.error("there was a problem restarting the system below the component: " + b.name);	
        			return false;
    			}
    		}
    	}

    	if(b instanceof IComponent) {
    		synchronized(b) {
	    		IComponent ic = (IComponent) b;
	    		ic.reactivate();
    		}
    	}
    		
		logger.debug("Component/Connector: " + b.name + " is now revived");    	
    	
    	return true;
	}
}
