package Prism.extensions.component.deployment;

import java.io.File;
import java.io.IOException;
import java.net.*;
import java.io.FileOutputStream;
import java.lang.reflect.InvocationTargetException;

import Prism.core.*;
import Prism.extensions.component.*;
import Prism.extensions.port.*;
import Prism.extensions.port.distribution.*;


public class Admin extends AbstractImplementation
{
	String hostAddr;		// IP address of a host running the Desi server
	
	public Admin(String pHostAddr)
	{
		hostAddr = pHostAddr;
	}
	
	public void start()
	{		
	}
	
	public void handle(Event e)
	{	
		String eventIP = (String) e.getParameter(String.valueOf(PrismConstants.UNICAST_DST_IP));	
		try {
			//System.out.println("received ip is: " + eventIP);
			//System.out.println("InetAddress ip is: " + InetAddress.getLocalHost().getHostAddress());
			if (eventIP.equals(InetAddress.getLocalHost().getHostAddress()))	// if the message is intended for this host
			{
				switch (Integer.valueOf(e.name).intValue()) 
				{
					case PrismConstants.CMD_RECEIVE_FILE:
					{
						//System.out.println("received CMD_RECEIVE_FILE");
						processCmdReceiveFile(e);
						break;
					}
					case PrismConstants.CMD_RECEIVE_COMP:	// Receive Component Command
					{
						//System.out.println("received CMD_RECEIVE_COMP");				
						processCmdReceiveComp(e);
						break;
					}
					case PrismConstants.CMD_INSTAN_COMP:		// Instantiate Component Command
					{
						//System.out.println("received CMD_INSTAN_COMP");				
						processCmdInstanComp(e);
						break;
					}
					case PrismConstants.CMD_INSTAN_PORT:	// Instantiate Port Command	
					{
						//System.out.println("received CMD_INSTAN_PORT");								
						processCmdInstanPort(e);
						break;
					}
					case PrismConstants.CMD_INSTAN_CONN:		// Instantiate Connector Command
					{
						//System.out.println("received CMD_INSTAN_CONN");				
						processCmdInstanConn(e);
						break;
					}
					case PrismConstants.CMD_WELD_PORT: // Weld Mutual Ports Command
					{
						//System.out.println("received CMD_WELD_PORT");				
						processCmdWeldPort(e);
						break;
					}
					case PrismConstants.CMD_ARCH_START: // Architecture Start Command
					{	
						//System.out.println("received CMD_ARCH_START");				
						processCmdArchStart();
						break;
					}
					case PrismConstants.CMD_EXTPORT_CONNECT: // ExtensiblePort Connect Command
					{
						//System.out.println("received CMD_EXTPORT_CONNECT");				
						processCmdExtPortConnect(e);
						break;
					}
					/*case PrismConstants.CMD_BRICK_START: // Brick Start Command
					{
						processCmdBrickStart(e);
						break;
					}
					case PrismConstants.CMD_SEND_COMP:		// Send Component Command
						break;
					case PrismConstants.CMD_UNWELD_PORT:	// Unweld Mutual Ports Comamnd
						break;	
					case PrismConstants.CMD_UNLOAD_COMP:	// Unload Component Command
						break;*/
				}		
				sendACK();
			}
		}
		catch (Exception exc) {			
			exc.printStackTrace();
		}
	}

	public void processCmdReceiveFile (Event event)
	{
		String fileName = (String)event.getParameter(String.valueOf(PrismConstants.COMP_FILE_NAME));
		String path = (String)event.getParameter(String.valueOf(PrismConstants.PATH));
		byte [] fileContent = (byte []) event.getParameter(String.valueOf(PrismConstants.DLL_FILE));		
		File file = new File(path+fileName);
		File dir = new File("\\Program Files\\Testing Deployer\\"+path);
		//File dir = new File(path);		
		try {
				dir.mkdirs();
				FileOutputStream out = new FileOutputStream(file);
				out.write(fileContent);
				out.flush();
				out.close();
		} catch (IOException ex) {
			ex.printStackTrace();
		}
	}
	
	public void processCmdReceiveComp(Event event)
	{
		
		byte [] fileContent = (byte [])event.getParameter(String.valueOf(PrismConstants.DLL_FILE));
		String fileName = (String)event.getParameter(String.valueOf(PrismConstants.COMP_FILE_NAME));
		File file = null;
		File path = null;
		if (!fileName.substring(fileName.length()-3, fileName.length()).equals("Imp"))
		{
			file = new File("\\Program Files\\Testing Deployer\\tdspp\\" + fileName + "\\" + fileName+".class");
			path = new File ("\\Program Files\\Testing Deployer\\tdspp\\" + fileName);
			//file = new File("tdspp\\" + fileName + "\\" + fileName+".class");
			//path = new File ("tdspp\\" + fileName);
			
		}
		else
		{
			file = new File("\\Program Files\\Testing Deployer\\tdspp\\" + fileName.substring(0, fileName.length()-3) + "\\" + fileName+".class");			
			path = new File("\\Program Files\\Testing Deployer\\tdspp\\" + fileName.substring(0, fileName.length()-3));
			//file = new File("tdspp\\" + fileName.substring(0, fileName.length()-3) + "\\" + fileName+".class");			
			//path = new File("tdspp\\" + fileName.substring(0, fileName.length()-3));
			
		}
        if(!(file.exists())) {
           try{           		
           		System.out.println("writing file: " + file.getName());
           		path.mkdirs();
                FileOutputStream out = new FileOutputStream(file);
                out.write(fileContent);
           		out.flush();
           		out.close();
           }
           catch(IOException ex) {
           		System.out.println("Error while writing the file in Admin.processCmdReceiveComp.");
           		ex.printStackTrace();
           }
        }		
	} 
		
	void processCmdInstanComp(Event event)
	{    					
		AbstractImplementation componentImp = null;
	    Object [] parameters = null;
        Class [] cl = null;
	    String basicCompFileName = (String) event.getParameter(String.valueOf(PrismConstants.COMP_FILE_NAME));
	    String compFileName = null;
	    if (basicCompFileName.equals("hq.RenderingAgent"))
	    	compFileName = "tdspp." + basicCompFileName + ".RenderingAgent";
	    else if (basicCompFileName.equals("RenderingAgent1_1") || basicCompFileName.equals("RenderingAgent2_4"))
	    	compFileName = "tdspp." + "RenderingAgent.RenderingAgent";
	    else
	    	compFileName = "tdspp." + basicCompFileName + "." + basicCompFileName;
		Integer compIDNameInt = (Integer)event.getParameter(String.valueOf(PrismConstants.COMP_NAME));
		String compIDName = compIDNameInt.toString();	    	    
        try {        	
        	if (basicCompFileName.equals("RenderingAgent1_1") || basicCompFileName.equals("RenderingAgent2_4"))
        	{
        		cl = new Class [2];
				cl[0] = Class.forName("java.lang.String");
        		cl[1] = Class.forName("java.lang.Integer");
        		parameters = new Object[2];
        		parameters[0] = compIDName;
        		if (basicCompFileName.equals("RenderingAgent1_1"))
        			parameters[1] = new Integer (0);
        		else if (basicCompFileName.equals("RenderingAgent2_4"))
        			parameters[1] = new Integer (7);
        		componentImp = (AbstractImplementation)Class.forName(compFileName).getConstructor(cl).newInstance(parameters);
        	}
        	else
        	{
        		cl = new Class [1];
				cl[0] = Class.forName("java.lang.String");        		
        		parameters = new Object[1];
        		parameters[0] = compIDName;
        		try {
        			componentImp = (AbstractImplementation)Class.forName(compFileName).getConstructor(cl).newInstance(parameters);
        		} catch (InvocationTargetException ex) 
				{
        			System.out.println(ex.getTargetException());
        			System.out.println(ex.getMessage());        			
				}
        	}
		}catch(Exception e) {
			e.printStackTrace();
			System.out.println(e);
		}    				
        Component component = new Component(compIDName, componentImp);
        componentImp.setAssociatedComponent(component);
        
        Architecture arch = (Architecture)((ExtensibleComponent)getAssociatedComponent()).getArchitecture();
		component.scaffold = arch.scaffold;
        arch.add(component);		
	}		

	void processCmdInstanConn(Event event)
	{
		Integer connNameInt = (Integer) event.getParameter(String.valueOf(PrismConstants.CONN_NAME));
		String connName = connNameInt.toString();
		Connector connector = new Connector(connName);
        Architecture arch = (Architecture)((ExtensibleComponent)getAssociatedComponent()).getArchitecture();		
		connector.scaffold = arch.scaffold;
		arch.add(connector);	
	}

	void processCmdInstanPort(Event event)
	{
		Architecture arch = (Architecture)((ExtensibleComponent)getAssociatedComponent()).getArchitecture();
		
		Port port = null;
		String portName = null;
		
		// create a basic request port
		if(event.hasParameter(String.valueOf(PrismConstants.CREATE_BASIC_REQUEST_PORT)))
		{
			Integer portNameInt = (Integer) event.getParameter(String.valueOf(PrismConstants.CREATE_BASIC_REQUEST_PORT));
			portName = portNameInt.toString();
			port = new Port(portName, PrismConstants.REQUEST);
		}
		// create a basic reply port 
		else if(event.hasParameter(String.valueOf(PrismConstants.CREATE_BASIC_REPLY_PORT)))
		{
			Integer portNameInt =  (Integer)(event.getParameter(String.valueOf(PrismConstants.CREATE_BASIC_REPLY_PORT)));
			portName = portNameInt.toString();
			port = new Port(portName, PrismConstants.REPLY);
		}
		// create a reply extensible port
		else if (event.hasParameter(String.valueOf(PrismConstants.CREATE_DIST_SERVER_PORT)))
		{
			Integer portNameInt = (Integer) event.getParameter(String.valueOf(PrismConstants.CREATE_DIST_SERVER_PORT));
			portName = portNameInt.toString();
			int portNum = ((Integer) event.getParameter(String.valueOf(PrismConstants.SERVER_PORT_NUM))).intValue();			
			ExtensiblePort ep = new ExtensiblePort(portName, PrismConstants.REPLY);
			SocketDistribution sd = new SocketDistribution(ep, portNum);
			ep.addDistributionModule(sd);			
			port = ep;
		}
		// create a request extensible port
		else if(event.hasParameter(String.valueOf(PrismConstants.CREATE_DIST_CLIENT_PORT)))
		{
			Integer portNameInt = (Integer) event.getParameter(String.valueOf(PrismConstants.CREATE_DIST_CLIENT_PORT));
			portName = portNameInt.toString();
			ExtensiblePort ep = new ExtensiblePort(portName, PrismConstants.REQUEST);    
			SocketDistribution sd = new SocketDistribution(ep);    
			ep.addDistributionModule(sd);			
			port = ep;
		}
		
		port.scaffold = arch.scaffold;
		// add the created port to the architecture
		arch.add(port);
		
		// Attach the created port to a component or a connector							
		if(event.hasParameter(String.valueOf(PrismConstants.PARENT_COMP)))
		{
			Integer parentBrickNameInt = (Integer) event.getParameter(String.valueOf(PrismConstants.PARENT_COMP));
			String parentBrickName = parentBrickNameInt.toString();
			Component parentComp = (Component)arch.getBrickByInstanceName(parentBrickName);
			parentComp.addCompPort(port);
		}
		else if(event.hasParameter(String.valueOf(PrismConstants.PARENT_CONN)))
		{
			Integer parentBrickNameInt = (Integer) event.getParameter(String.valueOf(PrismConstants.PARENT_CONN));
			String parentBrickName = parentBrickNameInt.toString();
			Connector parentConn = (Connector)arch.getBrickByInstanceName(parentBrickName);
			parentConn.addConnPort(port);
		}
		
	}

	void processCmdWeldPort(Event event)
	{		
		// get the two ports' names to be welded
		Integer reqPortNameInt = (Integer) event.getParameter(String.valueOf(PrismConstants.REQ_PORT_NAME));
		String reqPortName = reqPortNameInt.toString();
		Integer repPortNameInt = (Integer) event.getParameter(String.valueOf(PrismConstants.REP_PORT_NAME));
		String repPortName = repPortNameInt.toString();
		
		Architecture arch = (Architecture)((ExtensibleComponent)getAssociatedComponent()).getArchitecture();
		
		// get the two ports from the architecture
		Port reqPort = (Port) arch.getBrickByInstanceName(reqPortName);
		Port repPort = (Port) arch.getBrickByInstanceName(repPortName);
		
		// weld the ports
		arch.weld(reqPort, repPort);
		
	}

	void processCmdArchStart()
	{		
		Architecture arch = (Architecture)((ExtensibleComponent)getAssociatedComponent()).getArchitecture();
		arch.start();
	}

	void processCmdExtPortConnect(Event event)
	{		
		// get the ExtensiblePort name
		Integer portNameInt = (Integer) event.getParameter(String.valueOf(PrismConstants.EXT_PORT_NAME));
		String portName = portNameInt.toString();
		// get the IP address of the server
		String servAddr = (String) event.getParameter(String.valueOf(PrismConstants.SERVER_ADDR));
		// get the port number
		int portNum = ((Integer) event.getParameter(String.valueOf(PrismConstants.SERVER_PORT_NUM))).intValue();
		
		Architecture arch = (Architecture)((ExtensibleComponent)getAssociatedComponent()).getArchitecture();
		// get the ExtensiblePort object from the architecture
		ExtensiblePort extPort = (ExtensiblePort) arch.getBrickByInstanceName(portName);
		// connect to the server
		extPort.connect(servAddr, portNum);		
	}

	/*void Admin::processCmdBrickStart(Event* ap_event)
	{
		ass_MethodStart("Admin::processCmdBrickStart", nil);
		
		int* compName = (int *) ap_event->getParameter(PrismConstants::COMP_NAME);
		
		Architecture* ptr_arch = (Architecture *) getArchReflection();
		Component* ptr_comp = (Component *) ptr_arch->getBrickByInstanceName(*compName);
		ptr_comp->getImplementation()->start();
		
		ass_MethodFinishNoRV("Admin::processCmdBrickStart", nil);
	}*/	
	
	void sendACK()
	{
		// create the ACK event	

		Event ackEvent = new Event(String.valueOf(PrismConstants.DEPLOYMENT_ACK));
		ackEvent.eventType = PrismConstants.REQUEST;
		ackEvent.addParameter(String.valueOf(PrismConstants.UNICAST_DST_IP), this.hostAddr);
		
		// send the ACK event
		send(ackEvent);		
	}	

}
