package Prism.extensions.port.distribution;

import Prism.core.*;
import java.io.*;
import java.net.*;

/**
 *Connection object used by the SocketDistribution to connect.
 */ 

public class Connection     extends AbstractConnection //extends Brick implements IPortNames
{
	
	private ObjectOutputStream writer;

	/**
     * Thread that sits at the socket for this connection and creates new events when
     * they arrive
     */
    private ReadConnectionThread readConn;
        
    public Connection (AbstractDistribution p, String hostName, int portNum)
    {
            parent=p;
            try
            {
                socket=new Socket(hostName,portNum);
                writer = new ObjectOutputStream(socket.getOutputStream());

                System.out.println("local port:"+socket.getLocalPort());
                readConn = new ReadConnectionThread(this,  socket);
                readConn.start();
            }
            catch (Exception e)
            {
                System.out.println("\nException in Connection: constructor ->" + e.toString());
                //this.reconnect();
            }
	}

    public Connection( AbstractDistribution p, Socket s)
	{
		parent = p;
		socket=s;
		try
		{
                    writer = new ObjectOutputStream(socket.getOutputStream());

                    readConn = new ReadConnectionThread(this,  socket);
                    readConn.start();
                }
		catch (IOException e)
                {
			System.out.println("\nException in Connection: constructor ->" + e.toString());
			//this.reconnect();
                }
	}

	public synchronized void writeEvent(Object eventObject)
	{
		try
		{
					//System.out.println("sending event: "+((Event)eventObject).name);
					writer.writeObject(eventObject);
					//System.out.println("finishing transmission ...");
                    writer.flush();
                    writer.reset();
                    //System.out.println("Sent event: " + ((Event)eventObject).name);
		}
		catch(IOException ioe)
		{
                    ioe.printStackTrace ();
                    this.close();
                    //this.reconnect();
		}
		catch(Exception e1)
		{
                    e1.printStackTrace();
                    System.out.println("%%%%%error in writing event");
                    this.close();
                    //this.reconnect();			
                }
	}
        
 	public void readEvent(Object eventObject)
 	{
                //System.out.println("Received event: " + ((Event)eventObject).name);
                parent.readEvent(eventObject); 
	}        

    	public void close()
    	{
                try
		{
                    writer.close();
                    readConn.stopWorking();
                    parent.removeConnection(this);
		}
		catch (Exception e)
	    	{
                    System.out.println(" Connection: Caught exception in close() " + e.toString());
	    	}
    	}

	public void teardown()
    	{
    		try
		{
			writer.close();
			readConn.stopWorking();
		}
		catch (Exception e)
	    	{
	    		System.out.println("TBC Connection: Caught exception in teardown() " + e.toString());
	    	}
    	}



}



