package Prism.extensions.port.distribution;

import Prism.core.*;
import java.io.*;
import java.net.*;

/**
 *Connection object used by the Java2CPPSocketDistribution to connect.
 */ 

public class Java2CPPConnection     extends AbstractConnection //extends Brick implements IPortNames
{
	int bufSize = 64000;
	
	private BufferedOutputStream writer;

	/**
     * Thread that sits at the socket for this connection and creates new events when
     * they arrive
     */
    private Java2CPPReadConnectionThread readConn;
        
    public Java2CPPConnection (AbstractDistribution p, String hostName, int portNum)
    {
            parent=p;
            try
            {
                socket=new Socket(hostName,portNum);
                writer = new BufferedOutputStream(socket.getOutputStream(), bufSize);

                System.out.println("local port:"+socket.getLocalPort());
                readConn = new Java2CPPReadConnectionThread(this,  socket);
                readConn.start();
            }
            catch (Exception e)
            {
                System.out.println("\nException in Connection: constructor ->" + e.toString());
                //this.reconnect();
            }
	}

    public Java2CPPConnection( AbstractDistribution p, Socket s)
	{
		parent = p;
		socket=s;
		try
		{
            writer = new BufferedOutputStream(socket.getOutputStream(), bufSize);

            readConn = new Java2CPPReadConnectionThread(this,  socket);
            readConn.start();
        }
		catch (IOException e)
        {
			System.out.println("\nException in Connection: constructor ->" + e.toString());
			//this.reconnect();
        }
	}
    
    public Java2CPPConnection (AbstractDistribution p, String hostName, int portNum, int bufferSize)
    {
    	parent=p;
    	bufSize = bufferSize;
    	
        try
        {
            socket=new Socket(hostName,portNum);
            writer = new BufferedOutputStream(socket.getOutputStream(), bufSize);

            System.out.println("local port:"+socket.getLocalPort());
            readConn = new Java2CPPReadConnectionThread(this,  socket);
            readConn.start();
        }
        catch (Exception e)
        {
            System.out.println("\nException in Connection: constructor ->" + e.toString());
            //this.reconnect();
        }
	}
    
    public Java2CPPConnection( AbstractDistribution p, Socket s, int bufferSize)
    {
    	parent = p;
		socket=s;
		bufSize = bufferSize;
		
		try
		{
            writer = new BufferedOutputStream(socket.getOutputStream(), bufSize);

            readConn = new Java2CPPReadConnectionThread(this,  socket);
            readConn.start();
        }
		catch (IOException e)
        {
			System.out.println("\nException in Connection: constructor ->" + e.toString());
	        //this.reconnect();
        }
    }

	public synchronized void writeEvent(Object eventObject)
	{		
		try
		{	
			short segmentID=0, numOfSegments=0;
			int eventSize=0, segmentSize = bufSize + 4;
			ByteArrayOutputStream byteStream = new ByteArrayOutputStream(segmentSize);
			DataOutputStream out = new DataOutputStream(byteStream);
			
			byte[] eventStream = convertEventToByteArray((Event)eventObject);
			eventSize = eventStream.length;
			
			//System.out.println("sending event: "+((Event)eventObject).name);
			
			if(eventSize <= bufSize)
			{
				segmentID = 1;
				numOfSegments = 1;
				
				out.writeShort(segmentID);
				out.writeShort(numOfSegments);
				out.write(eventStream);
				
				byte[] eventSegment = byteStream.toByteArray();
				byte[] segmentBuffer = new byte[segmentSize];
				System.arraycopy(eventSegment, 0, segmentBuffer, 0, eventSegment.length);
				writer.write(segmentBuffer);
				writer.flush();
				out.close();
			}
			else
			{
				byte[] segmentBuffer = new byte[segmentSize];
				
				if((eventSize % bufSize) > 0)
					numOfSegments = (short) (eventSize/bufSize + 1);
				else
					numOfSegments = (short) (eventSize/bufSize);
				
				for(int pos=0; pos < eventSize;)
				{
					segmentID++;
					out.writeShort(segmentID);
					
					out.writeShort(numOfSegments);
					
					if(pos+bufSize <= eventSize)
					{
						out.write(eventStream, pos, bufSize);
						pos += bufSize;
					}
					else
					{
						out.write(eventStream, pos, eventSize-pos);
						pos = eventSize;
					}
					
					// send an event segment out over the TCP socket
					byte[] eventSegment = byteStream.toByteArray();
					
					System.arraycopy(eventSegment, 0, segmentBuffer, 0, eventSegment.length);
					writer.write(segmentBuffer);
					writer.flush();
					
					out.flush();
					byteStream.reset();
				}
				out.close();
			}
            
		}
		catch(IOException ioe)
		{
            ioe.printStackTrace ();
            this.close();
            //this.reconnect();
		}
		catch(Exception e1)
		{
            e1.printStackTrace();
            System.out.println("%%%%%error in writing event");
            this.close();
            //this.reconnect();			
        }
	}
        
 	public void readEvent(Object eventObject)
 	{
        //System.out.println("Received event: " + ((Event)eventObject).name);
        parent.readEvent(eventObject); 
	}        

	public void close()
	{
        try
		{
	                writer.close();
	                readConn.stopWorking();
	                parent.removeConnection(this);
		}
		catch (Exception e)
    	{
                System.out.println(" Connection: Caught exception in close() " + e.toString());
    	}
	}

	public void teardown()
	{
		try
		{
			writer.close();
			readConn.stopWorking();
		}
		catch (Exception e)
    	{
    		System.out.println("TBC Connection: Caught exception in teardown() " + e.toString());
    	}
	}
	
	public byte[] convertEventToByteArray(Event eventObject) throws IOException 
	{
		int eventObjectSize=0, vectorObjectSize=0;
		int paramNum = eventObject.parameterName.size();
		
		int[] payloadName = new int[paramNum];
		int[] payloadType = new int[paramNum];
		Object[] payloadValue = new Object[paramNum];
		int[] payloadSize = new int[paramNum];
		
		for(int i=0; i < paramNum; i++)
		{
			payloadName[i] = Integer.parseInt((String)eventObject.parameterName.elementAt(i));
			payloadValue[i] = eventObject.parameterValue.elementAt(i);
			if(payloadValue[i] instanceof Integer)
			{
				payloadType[i] = Java2CPPConstants.PAYLOAD_INT;
				payloadSize[i] = 18; // header size(6 bytes) + sizeof(int m_name) + sizeof(int m_type) + 
				                     // sizeof(int mp_value)
			}
			else if(payloadValue[i] instanceof Byte)
			{
				payloadType[i] = Java2CPPConstants.PAYLOAD_CHAR;
				payloadSize[i] = 15; // header size(6 bytes) + sizeof(int m_name) + sizeof(int m_type) + 
				                     // sizeof(char mp_value)
			}
			else if(payloadValue[i] instanceof String)
			{
				payloadType[i] = Java2CPPConstants.PAYLOAD_STRING;
				payloadSize[i] = 18 + ((String)payloadValue[i]).length(); // header size(6 bytes) + sizeof(int m_name) + 
				                                                       // sizeof(int m_type) + sizeof(int stringSize) + 
																	   // strlen(char* mp_value)
			}
			else if(payloadValue[i] instanceof Float)
			{
				payloadType[i] = Java2CPPConstants.PAYLOAD_FLOAT;
				payloadSize[i] = 18; // header size(6 bytes) + sizeof(int m_name) + sizeof(int m_type) + 
				                     // sizeof(float mp_value)
			}
			else 
			{
				payloadType[i] = Java2CPPConstants.PAYLOAD_OBJECT;
				
				//@todo: need to define DLLFile class and its getObectSize method
				//payloadSize[i] = 14 + ((DLLFile)payloadValue[i]).getObjectSize(); // header size(6 bytes) + sizeof(int m_name) + 
																					 //sizeof(int m_type) + sizeof(DLLFile object)
				
				if(payloadValue[i] instanceof File)
				{
					File dllFile = (File)payloadValue[i];
					int fileSize = (int) dllFile.length();
					String fileName = dllFile.getName();
					
					// header size(6 bytes) + sizeof(int m_name) + sizeof(int m_type) + header size(4 bytes) 
					// + sizeof(int m_fileNameSize) + strlen(char* mp_fileName) + sizeof (int m_fileSize) + sizeof(DLL file)
					payloadSize[i] = 14 + 4 + 4 + fileName.length() + 4 + fileSize; 
				}
			}
			
			vectorObjectSize += payloadSize[i];
		}
		
		vectorObjectSize += 10; // header size(6 bytes) + sizeof(int m_VectorSize) + size of all payloads
		eventObjectSize = 14 + vectorObjectSize; // header size(6 bytes) + sizeof(int m_name) + sizeof(int m_eventType) + size of a vector object
		
		ByteArrayOutputStream bytestream = new ByteArrayOutputStream(eventObjectSize);
		DataOutputStream out = new DataOutputStream(bytestream);
		
		out.writeInt(eventObjectSize); // event object size
		out.writeShort(Java2CPPConstants.EVENT); // event typeID
		out.writeInt(Integer.parseInt(eventObject.name)); // event name
		out.writeInt(eventObject.eventType); // event type
		out.writeInt(vectorObjectSize); // vector object size 
		out.writeShort(Java2CPPConstants.FW_VECTOR); // vector typeID
		out.writeInt(paramNum); // number of elements in the vector object
		for(int i=0; i < paramNum; i++)
		{
			out.writeInt(payloadSize[i]); // payload object size
			out.writeShort(Java2CPPConstants.FW_PAYLOAD); // payload typeID
			out.writeInt(payloadName[i]); // payload name
			out.writeInt(payloadType[i]); // payload type
			switch(payloadType[i])
			{
				case Java2CPPConstants.PAYLOAD_INT: {
					out.writeInt(((Integer)payloadValue[i]).intValue());
					break;
				}
				case Java2CPPConstants.PAYLOAD_CHAR: {
					out.write(((Byte)payloadValue[i]).byteValue());
					break;
				}
				case Java2CPPConstants.PAYLOAD_STRING: {
					String str = (String)payloadValue[i];
					int strLength = str.length();
					out.writeInt(strLength);
					for (int j=0; j<strLength; j++)
						out.write((byte) str.charAt(j));
					break;
				}
				case Java2CPPConstants.PAYLOAD_FLOAT: {
					out.writeFloat(((Float)payloadValue[i]).floatValue());
					break;
				}
				case Java2CPPConstants.PAYLOAD_OBJECT: {
					//@todo: need to implement the toByteArray method of DLLFile class
					//out.write(((DLLFile)payloadValue[i]).toByteArray());
					if(payloadValue[i] instanceof File)
					{
						File dllFile = (File)payloadValue[i];
						int fileSize = (int) dllFile.length();
						String fileName = dllFile.getName();
						int fileNameSize = fileName.length();
						
						out.writeInt(payloadSize[i]-14);
						out.writeInt(fileNameSize);
						for(int j=0; j < fileNameSize; j++)
							out.write((byte)fileName.charAt(j));
						
						out.writeInt(fileSize);
						FileInputStream fis = new FileInputStream(dllFile);
						DataInputStream dis = new DataInputStream(fis);
						byte[] buffer = new byte[fileSize];
						dis.read(buffer);
						out.write(buffer);
						dis.close();
					}
					break;
				}
			}
		}
		
		byte byte_array[] = bytestream.toByteArray();
		out.close();
		
		return byte_array;
	}
}



