
package Prism.extensions.port.distribution;

import java.io.*;
import java.net.*;
import Prism.core.*;

/**
 * ReadConnectionThread class is used to read the socket continuosly.
 *
 *@version 2.0
 *@author USC Soft. Arch. Group. Contact: Sam Malek <A HREF="mailto:malek@usc.edu"> malek@usc.edu </A>
 */ 
 class Java2CPPReadConnectionThread extends Thread
{
 	static int BUFFSIZE = 64004;
 	
    /**
    * ReadConnectionThread will keep on running as long as keepWorking is true.
    */
     protected boolean keepWorking;
    
    /**
    * Connection to which this thread belongs
    */
     private Java2CPPConnection conn;
    
    /**
     * The StreamConnection class is used to instantiate a socket on a particular port
     */	
    private Socket streamConn;
		 
    private BufferedInputStream ir;
    
    private byte[] data;
    
    /** 
    * This constructor instantiates a ReadConnectionThread on a particular socket.
    * @params parent  A connector that drives the ReadConnectionThread
    * @params socket  The socket on which the connection is made
    */
    public Java2CPPReadConnectionThread(Java2CPPConnection conn,  Socket sc)
    {
        super();
		keepWorking = true;
		this.conn = conn;
		streamConn = sc;
	
		try
		{
	            ir = new BufferedInputStream(streamConn.getInputStream(),BUFFSIZE);
	            data = new byte[BUFFSIZE];
		}
		catch (IOException e)
		{
	            e.printStackTrace ();
		}

    }


    public void run()
    {
        Object eventObject = null;
        int numBytes;
        
    	try
    	{
            while (keepWorking)
            {
            	//eventObject = (Object) ir.readObject();
            	numBytes = ir.read(data);
            	eventObject = ConvertByteArrayToEvent(numBytes);  
            	
            	/*Event e = (Event) eventObject;
            	Integer temp = (Integer)e.getParameter(String.valueOf(PrismConstants.COMMAND_NUM));
            	if(temp != null)
            		System.out.println("In ReadConnectionThread ACK ID: " + temp.intValue());*/
            	conn.readEvent(eventObject);
            } //while ends

            finalize1();
        }
  	catch (Exception e1)
  	{
            System.out.println("Caught Exception in Java2CPPReadConnectionThread " + e1.toString());
            conn.close();
  	} // catch ends
    }

    void finalize1()
    {
	try
	{
            ir.close();
            streamConn.close();
	}
	catch (IOException e){}
    }

    public void stopWorking()
    {
    	keepWorking = false;
    }
    
    public Object ConvertByteArrayToEvent(int numBytes) throws IOException
    {
    	int evtName, evtType, vectorSize, payloadName, payloadType;
    	Event event = new Event();
    	    	
    	ByteArrayInputStream bytestream = new ByteArrayInputStream(data);
		DataInputStream instream = new DataInputStream(bytestream);

		instream.skipBytes(10);
		evtName = instream.readInt();
	
		/*
		if(evtName == PrismConstants.DEPLOYMENT_REQ)
		{
			event.name = "Deployment_Request";
		}
		else if(evtName == PrismConstants.DEPLOYMENT_ACK)
		{
			event.name = "Deployment_ACK";
		}
		*/
		
		event.name = String.valueOf(evtName);
		
		evtType = instream.readInt();
		event.eventType = evtType;
		
		instream.skipBytes(6);
		vectorSize = instream.readInt();
		
		for(int i=0; i < vectorSize; i++)
		{
			instream.skipBytes(6);
			payloadName = instream.readInt();
			payloadType = instream.readInt();
			
			//System.out.println("payload type and name: " + payloadType + ", " + payloadName);
			
			switch(payloadType) 
			{
				case Java2CPPConstants.PAYLOAD_INT : {
					event.addParameter((new Integer(payloadName)).toString(), new Integer(instream.readInt()));
					break;
				}
				case Java2CPPConstants.PAYLOAD_CHAR : {
					event.addParameter((new Integer(payloadName)).toString(), new Byte(instream.readByte()));
					break;
				}
				case Java2CPPConstants.PAYLOAD_STRING : {
					int strSize = instream.readInt();
					byte[] str = new byte[strSize];
					for(int j=0; j < strSize; j++)
					{
						str[j] = instream.readByte();
					}
					//System.out.println("payload value: " + new String(str));
					event.addParameter((new Integer(payloadName)).toString(),new String(str));
					break;
				}
				case Java2CPPConstants.PAYLOAD_FLOAT : {
					event.addParameter((new Integer(payloadName)).toString(), new Float(instream.readFloat()));
					break;
				}
				case Java2CPPConstants.PAYLOAD_OBJECT : {
					break;
				}
				default:
					break;
			}
		}
		
		instream.close();
		
    	return event;
    }
}
